class WhinesController < ApplicationController
  skip_before_filter :login_required, :only => [:index, :show]
  before_filter :find_user
  before_filter :parse_atom
  # for indicator demo
  #before_filter :delay_action
  
  def home
    @whine = Whine.new
    @whines = current_user.whines_with_friends(:limit => 20)
  end

  # GET /whines
  # GET /whines.atom
  def index
    if @user
      if logged_in? && !current_user.friend?(@user)
        @friendship = Friendship.new(:user => current_user,
                                     :friend => @user)
      end
      @whines = @user.whines.find(:all,
                                  :order => "whines.created_at desc",
                                  :limit => 20)
    else
      @whines = Whine.find(:all,
                           :order => "created_at desc",
                           :limit => 20)
    end
    respond_to do |format|
      format.html
      format.atom
    end
  end

  # GET /whines/1
  def show
    @whine = Whine.find(params[:id])
    respond_to do |format|
      format.html
      format.atom do
        headers["Content-Type"] = "application/atom+xml;type=entry"
      end
    end
  end

  # POST /whines
  def create
    @whine = Whine.new(params[:whine])
    @whine.user = current_user
    @whine.save
    respond_to do |format|
      format.js
      format.atom do
        if @whine.valid?
          headers["Content-Type"] = "application/atom+xml;type=entry"
          headers["Location"] = whine_url(@whine) + ".atom"
          render :action => "show", :status => 201
        else
          headers["Content-Type"] = "text/plain"
          render :text => "validation failed\n", :status => 400
        end
      end
    end
  end

  # DELETE /whines/1
  # DELETE /whines/1.xml
  def destroy
    @whine = current_user.whines.find(params[:id])
    @whine.destroy
    respond_to do |format|
      format.js
      format.atom
    end
  end

  private

  def find_user
    if params[:user_id]
      @user = User.find(params[:user_id])
    elsif params[:user_login]
      @user = User.find_by_login(params[:user_login])
    end
    return true
  end

  def parse_atom
    if /^application\/atom\+xml/.match(request.env["CONTENT_TYPE"])
      xml = REXML::Document.new(request.raw_post)
      params[:whine] = {
        :body => xml.elements["entry/content"].text
      }
    end
    return true
  end

  def delay_action
    sleep(3)
    return true
  end
end
